"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_native_1 = require("react-native");
var runTiming_1 = require("./Implementation/runTiming");
var createValue_1 = require("./Implementation/createValue");
var getProcessedColor_1 = require("./Implementation/getProcessedColor");
var getColorDisplayValue_1 = require("./Implementation/getColorDisplayValue");
var getProcessedRotation_1 = require("./Implementation/getProcessedRotation");
var utils_1 = require("./Implementation/utils");
var bezier_1 = require("./Implementation/bezier");
var RadNode_1 = require("./Implementation/RadNode");
var event = react_native_1.Animated.event;
var stackTrace = [];
var stackTraceToString = function () { return stackTrace.map(function (s) { return s.name; }); };
var gestureConfiguration = {};
var evalNode = function (input) {
    var retVal;
    if (isNode(input)) {
        retVal = input.evaluate();
    }
    else if (utils_1.isAnimatedNode(input)) {
        retVal = input.__getValue();
    }
    else if (typeof input === "function") {
        retVal = input();
    }
    else {
        if (typeof input === "string") {
            retVal = parseFloat(input);
        }
        else {
            retVal = input;
        }
    }
    return retVal;
};
var createBlockFunc = function (items) {
    return createNode("block", function () {
        var nodes = items.map(function (f) { return evalNode(f); });
        var last = nodes[nodes.length - 1];
        return last;
    }, items);
};
var createProxyNode = function () {
    var _argStack = new Array();
    var cache = {};
    return {
        _argStack: _argStack,
        beginCall: function (value) {
            _argStack.push(value);
        },
        endCall: function () {
            cache[_argStack.length] = undefined;
            _argStack.pop();
        },
        setValue: function (value) {
            // Cast and allow exception if not IAnimationNode
            _argStack[_argStack.length - 1].setValue(value);
        },
        __getValue: function () {
            if (!cache[_argStack.length]) {
                if (typeof _argStack[_argStack.length - 1] === "function") {
                    cache[_argStack.length] = _argStack[_argStack.length - 1]();
                }
                else if (isProxyNode(_argStack[_argStack.length - 1])) {
                    cache[_argStack.length] = _argStack[_argStack.length - 1].__getValue();
                }
                else {
                    cache[_argStack.length] = _argStack[_argStack.length - 1];
                }
            }
            return evalNode(cache[_argStack.length]);
        },
    };
};
function isProxyNode(value) {
    return (value &&
        Object.keys(value).find(function (k) { return k.startsWith("_argStack"); }) !== undefined);
}
exports.isProxyNode = isProxyNode;
var createNode = function (name, evaluate) {
    var children = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        children[_i - 2] = arguments[_i];
    }
    return {
        name: name,
        evaluate: function () {
            stackTrace.push({ name: name, children: children || [] });
            var node = evaluate();
            var retVal = evalNode(node);
            stackTrace.pop();
            return retVal;
        },
        children: children || [],
    };
};
var isNode = function (node) {
    return node && node.hasOwnProperty("evaluate");
};
var ReactNativeAnimationProvider = {
    getColorDisplayValue: getColorDisplayValue_1.getColorDisplayValue,
    getDisplayValue: function (input) { return input; },
    getRadianDisplayValue: function (input) {
        return new RadNode_1.RadNode(input);
    },
    createAnimatedComponent: react_native_1.Animated.createAnimatedComponent,
    runTiming: runTiming_1.runTiming,
    createValue: createValue_1.createValue,
    isAnimatedNode: utils_1.isAnimatedNode,
    getNumericColor: getProcessedColor_1.getProcessedColor,
    getNumericRotation: getProcessedRotation_1.getProcessedRotation,
    configuration: {
        gestureConfiguration: gestureConfiguration,
    },
    Types: {
        View: react_native_1.Animated.View,
        Image: react_native_1.Animated.Image,
        ScrollView: react_native_1.Animated.ScrollView,
        Text: react_native_1.Animated.Text,
    },
    Animated: {
        event: event,
        /* Operators */
        add: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return createNode.apply(void 0, __spreadArrays(["add",
                function () {
                    return args.reduce(function (a, b) { return evalNode(a) + evalNode(b); }, 0);
                }], args));
        },
        multiply: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return createNode.apply(void 0, __spreadArrays(["multiply",
                function () { return args.reduce(function (a, b) { return evalNode(a) * evalNode(b); }, 1); }], args));
        },
        divide: function (a, b) {
            return createNode("divide", function () { return evalNode(a) / evalNode(b); }, a, b);
        },
        pow: function (a, b) {
            return createNode("pow", function () { return Math.pow(evalNode(a), evalNode(b)); }, a, b);
        },
        exp: function (a) {
            return createNode("exp", function () { return Math.exp(evalNode(a)); }, a);
        },
        sqrt: function (a) {
            return createNode("sqrt", function () { return Math.sqrt(evalNode(a)); }, a);
        },
        sin: function (a) {
            return createNode("sin", function () { return Math.sin(evalNode(a)); }, a);
        },
        cos: function (a) {
            return createNode("cos", function () { return Math.cos(evalNode(a)); }, a);
        },
        sub: function (a, b) {
            return createNode("sub", function () { return evalNode(a) - evalNode(b); }, a, b);
        },
        round: function (input) {
            return createNode("round", function () { return Math.round(evalNode(input)); }, input);
        },
        eq: function (left, right) {
            return createNode("eq", function () { return evalNode(left) === evalNode(right); }, left, right);
        },
        neq: function (left, right) {
            return createNode("neq", function () { return evalNode(left) !== evalNode(right); }, left, right);
        },
        abs: function (a) { return Math.abs(evalNode(a)); },
        and: function (left, right) {
            // @ts-ignore
            var others = [];
            for (
            // @ts-ignore
            var _i = 2; 
            // @ts-ignore
            _i < arguments.length; 
            // @ts-ignore
            _i++) {
                // @ts-ignore
                others[_i - 2] = arguments[_i];
            }
            return createNode.apply(void 0, __spreadArrays(["and",
                function () {
                    var nodes = __spreadArrays([left, right], others);
                    var retVal = evalNode(left);
                    for (var i = 1; i < nodes.length; i++) {
                        retVal = retVal && evalNode(nodes[i]);
                    }
                    return retVal;
                }], __spreadArrays([left, right], others)));
        },
        greaterOrEq: function (left, right) {
            return createNode("greaterOrEq", function () { return evalNode(left) >= evalNode(right); }, left, right);
        },
        lessThan: function (left, right) {
            return createNode("lessThan", function () { return evalNode(left) < evalNode(right); }, left, right);
        },
        greaterThan: function (left, right) {
            return createNode("greaterThan", function () { return evalNode(left) > evalNode(right); }, left, right);
        },
        lessOrEq: function (left, right) {
            return createNode("lessOrEq", function () { return evalNode(left) <= evalNode(right); }, left, right);
        },
        /* Statements */
        set: function (target, source) {
            return createNode("set", function () {
                var v = evalNode(source);
                if (Number.isNaN(v)) {
                    var s = stackTrace;
                    throw new Error("Value is not a number\n" + stackTraceToString() + "\n" + s);
                }
                target.setValue(v);
                return v;
            }, target, source);
        },
        block: function (items) { return createBlockFunc(items); },
        cond: function (expression, ifNode, elseNode) {
            var ifNodeResolved = ifNode instanceof Array ? createBlockFunc(ifNode) : ifNode;
            var elseNodeResolved = elseNode instanceof Array ? createBlockFunc(elseNode) : elseNode;
            return createNode.apply(void 0, __spreadArrays(["cond",
                function () {
                    var expr = evalNode(expression);
                    var exec = expr ? ifNodeResolved : elseNodeResolved || 0;
                    var retVal = evalNode(exec);
                    return retVal;
                }], (elseNode ? [expression, ifNode, elseNode] : [expression, ifNode])));
        },
        call: function (args, callback) {
            return createNode.apply(void 0, __spreadArrays(["call",
                function () {
                    callback(args.map(function (a) { return evalNode(a); }));
                    return 0;
                }], args));
        },
        /* Helpers */
        proc: function (name, callback) {
            // Create argument nodes
            var params = new Array(callback.length);
            for (var i = 0; i < params.length; i++) {
                params[i] = createProxyNode();
            }
            var func = callback.apply(void 0, params);
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return createNode.apply(void 0, __spreadArrays(["proc-(" + name + ")",
                    function () {
                        if (args.length !== params.length) {
                            throw Error("Expected " + params.length + " arguments, got " + args.length + ".");
                        }
                        params.forEach(function (p, i) { return p.beginCall(args[i]); });
                        var retVal = func.evaluate();
                        params.forEach(function (p) { return p.endCall(); });
                        return retVal;
                    }], args));
            };
        },
        always: function (cb) {
            return createNode("always", function () {
                // We'll just return the node in RN Animated - since it
                // doesn't have to be marked for evaluation.
                // @ts-ignore
                return cb();
            });
        },
        attach: function (source, node) {
            // Add listener
            var id = source.addListener(function () { return evalNode(node); });
            listeners[id] = node;
        },
        detach: function (source, node) {
            // Remove listener
            Object.keys(listeners).forEach(function (key) {
                if (listeners[key] === node) {
                    source.removeListener(key);
                }
            });
        },
        debug: function (msg, value) {
            return function debug() {
                console.log("****** " + msg + ": " + evalNode(value));
                return evalNode(value);
            };
        },
        bezier: function (x1, y1, x2, y2) {
            var bezierFunc = bezier_1.bezier(x1, y1, x2, y2);
            return function (t) {
                return createNode("bezier", function () {
                    return bezierFunc(evalNode(t));
                }, t);
            };
        },
    },
};
exports.ReactNativeAnimationProvider = ReactNativeAnimationProvider;
var listeners = {};
//# sourceMappingURL=index.js.map